#ifndef Stepnode_h
#define Stepnode_h

class NodeManager;
#include "AccelStepnode.h"

class Stepnode {
public:
    Stepnode(NodeManager* manager, uint8_t id);

    void moveTo(long absolute);

    void move(long relative);

    bool runSpeed();

    bool run();

    void setMaxSpeed(float speed);

    float maxSpeed();

    void setAcceleration(float acceleration);

    void setSpeed(float speed);

    float speed();

    long distanceToGo();

    long targetPosition();

    long currentPosition();

    void setCurrentPosition(long position);

    void stop();

    void setMinPulseWidth(unsigned int minWidth);

    bool state_enabled;

    bool state_endstop;

protected:
    typedef enum {
        DIRECTION_CCW = 0,  ///< Clockwise
        DIRECTION_CW  = 1   ///< Counter-Clockwise
    } Direction;

    void computeNewSpeed();

    void step(long step);
private:
    NodeManager*   _manager;
    // the ID of the node (0, N-1)
    uint8_t        _id;

    /// The current absolution position in steps.
    long           _currentPos;    // Steps

    /// The target position in steps. The Stepnode library will move the
    /// motor from the _currentPos to the _targetPos, taking into account the
    /// max speed, acceleration and deceleration
    long           _targetPos;     // Steps

    /// The current motos speed in steps per second
    /// Positive is clockwise
    float          _speed;         // Steps per second

    /// The maximum permitted speed in steps per second. Must be > 0.
    float          _maxSpeed;

    /// The acceleration to use to accelerate or decelerate the motor in steps
    /// per second per second. Must be > 0
    float          _acceleration;
    float          _sqrt_twoa; // Precomputed sqrt(2*_acceleration)

    /// The current interval between steps in microseconds.
    /// 0 means the motor is currently stopped with _speed == 0
    unsigned long  _stepInterval;

    /// The last step time in microseconds
    unsigned long  _lastStepTime;

    /// The minimum allowed pulse width in microseconds
    unsigned int   _minPulseWidth;

    /// The step counter for speed calculations
    long _n;

    /// Initial step size in microseconds
    float _c0;

    /// Last step size in microseconds
    float _cn;

    /// Min step size in microseconds based on maxSpeed
    float _cmin; // at max speed

    /// Current direction motor is spinning in
    bool _direction; // 1 == CW
};
#endif
